"use strict";

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _express = _interopRequireDefault(require("express"));

var _qrcode = _interopRequireDefault(require("qrcode"));

var _moveDecimalPoint = _interopRequireDefault(require("move-decimal-point"));

var _promiseWrap = _interopRequireDefault(require("./lib/promise-wrap"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var rpath = function rpath(name) {
  return _path["default"].join(__dirname, name);
};

module.exports = function (app, payListen) {
  app.set('url', process.env.URL || '/');
  app.set('static_url', process.env.STATIC_URL || app.settings.url + 'static/');
  app.set('view engine', 'pug');
  app.set('views', rpath('../views'));

  app.locals.formatMsat = function (msat) {
    return (0, _moveDecimalPoint["default"])(msat, -8) + ' mBTC';
  };

  _fs["default"].existsSync(rpath('www')) // comes pre-built in dist/www
  ? app.use('/static', _express["default"]["static"](rpath('www'))) : app.use('/static', require('stylus').middleware({
    src: rpath('../www'),
    serve: true
  })).use('/static', _express["default"]["static"](rpath('../www')));
  app.get('/checkout/:invoice', (0, _promiseWrap["default"])( /*#__PURE__*/function () {
    var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(req, res) {
      var opt;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              opt = req.invoice.metadata && req.invoice.metadata.checkout || {};

              if (!(req.invoice.status == 'paid' && opt.redirect_url)) {
                _context.next = 3;
                break;
              }

              return _context.abrupt("return", res.redirect(opt.redirect_url));

            case 3:
              if (!(req.invoice.status == 'unpaid')) {
                _context.next = 7;
                break;
              }

              _context.next = 6;
              return _qrcode["default"].toDataURL("lightning:".concat(req.invoice.payreq).toUpperCase(), {
                margin: 1
              });

            case 6:
              res.locals.qr = _context.sent;

            case 7:
              res.render('checkout', req.invoice);

            case 8:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));

    return function (_x, _x2) {
      return _ref.apply(this, arguments);
    };
  }())); // like /invoice/:invoice/wait, but user-accessible, doesn't reveal the full invoice fields,
  // and with a fixed timeout.

  app.get('/checkout/:invoice/wait', (0, _promiseWrap["default"])( /*#__PURE__*/function () {
    var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(req, res) {
      var paid, expired;
      return regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (!(req.invoice.status == 'paid')) {
                _context2.next = 2;
                break;
              }

              return _context2.abrupt("return", res.sendStatus(204));

            case 2:
              if (!(req.invoice.status == 'expired')) {
                _context2.next = 4;
                break;
              }

              return _context2.abrupt("return", res.sendStatus(410));

            case 4:
              _context2.next = 6;
              return payListen.register(req.invoice.id, 60000);

            case 6:
              paid = _context2.sent;
              expired = !paid && req.invoice.expires_at <= Date.now() / 1000;
              res.sendStatus(paid ? 204 : expired ? 410 : 402);

            case 9:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function (_x3, _x4) {
      return _ref2.apply(this, arguments);
    };
  }()));
  app.get('/checkout/:invoice/qr.png', (0, _promiseWrap["default"])( /*#__PURE__*/function () {
    var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(req, res) {
      return regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _qrcode["default"].toFileStream(res.type('png'), "lightning:".concat(req.invoice.payreq).toUpperCase());

            case 1:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3);
    }));

    return function (_x5, _x6) {
      return _ref3.apply(this, arguments);
    };
  }()));
};