"use strict";

// Memoize the results of `fn` for up to `ms` milliseconds.
// Currently only supports functions of a single argument.
var memoize = function memoize(ms, fn) {
  var cache = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : new Map();
  return function (arg) {
    var cached = cache.has(arg) && cache.get(arg);

    if (cached && cached[0] > Date.now()) {
      return cached[1];
    } else {
      var val = fn(arg);
      cache.set(arg, [Date.now() + ms, val]);
      val["catch"](function (_) {
        return cache["delete"](arg);
      });
      return val;
    }
  };
};

module.exports = memoize;